//
//  XingAPIBookmarks.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+Bookmarks.h"

@implementation XingAPI (Bookmarks)

#pragma mark - Public methods

- (RACSignal *)createBookmark:(NSString *)userID {
    NSParameterAssert(userID);
    NSString *path = [NSString stringWithFormat:@"users/%@/bookmarks/%@", self.session.xingUserID, userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self putRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)deleteBookmark:(NSString *)userID {
    NSParameterAssert(userID);
    NSString *path = [NSString stringWithFormat:@"users/%@/bookmarks/%@", self.session.xingUserID, userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self deleteRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadBookmarks {
    return [self loadBookmarksForUser:nil parameters:nil];
}

- (RACSignal *)loadBookmarks:(NSDictionary *)parameters {
    return [self loadBookmarksForUser:nil parameters:parameters];
}

#pragma mark - Private methods

- (RACSignal *)loadBookmarksForUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/bookmarks", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingObjectBookmarks *bookmarks = (XingObjectBookmarks *)response.resultObject;
            [user addBookmarks:bookmarks.items];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

@end
