//
//  XingAPIContactPath.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+ContactPath.h"

@implementation XingAPI (ContactPath)

#pragma mark - Public methods

- (RACSignal *)loadContactPathToUser:(NSString *)userID
                          parameters:(NSDictionary *)parameters {
    return [self loadContactPathFromUser:nil toUser:userID parameters:parameters];
}

- (RACSignal *)loadContactPathToUser:(NSString *)userID {
    return [self loadContactPathFromUser:nil toUser:userID parameters:nil];
}

#pragma mark - Private methods

- (RACSignal *)loadContactPathFromUser:(NSString *)userID
                                toUser:(NSString *)otherUserID
                            parameters:(NSDictionary *)parameters {
    NSParameterAssert(otherUserID);
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/network/%@/paths", userID, otherUserID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingObjectContactPaths *contactPaths = (XingObjectContactPaths *)response.resultObject;
            XingEntityUser *originUser = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingEntityUser *destinationUser = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:otherUserID inContext:context];

            for (XingEntityContactPath *contactPath in contactPaths.items) {
                contactPath.fromUser = originUser;
                contactPath.toUser = destinationUser;
            }

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

@end
