//
//  XingAPIContactRequests.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+ContactRequests.h"

@implementation XingAPI (ContactRequests)

#pragma mark - Public methods

- (RACSignal *)createContactRequest:(NSString *)userID {
    return [self createContactRequest:userID parameters:nil];
}

- (RACSignal *)createContactRequest:(NSString *)userID parameters:(NSDictionary *)parameters {
    NSParameterAssert(userID);
    NSString *path = [NSString stringWithFormat:@"users/%@/contact_requests", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self postRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadContactRequests:(NSDictionary *)parameters {
    return [self loadContactRequestsForUser:nil parameters:parameters];
}

- (RACSignal *)loadContactRequests {
    return [self loadContactRequestsForUser:nil parameters:nil];
}

#pragma mark - Private methods

- (RACSignal *)loadContactRequestsForUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/contact_requests", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            NSSet *result = [NSSet setWithArray:response.result.array];
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];

            NSMutableSet *contactRequests = [NSMutableSet set];
            for (XingEntityContactRequest *request in result) {
                request.outgoing = @NO;
                [contactRequests addObject:request.sender.userID];
            }

            NSMutableSet *toDelete = [NSMutableSet set];
            for (XingEntityContactRequest *request in user.contactRequests) {
                if ([contactRequests containsObject:request.sender.userID]) {
                    [toDelete addObject:request];
                }
            }

            for (XingEntityContactRequest *request in toDelete) {
                [user removeContactRequestsObject:request];
                [context deleteObject:request];
            }

            [user addContactRequests:result];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

@end
