//
//  XingAPIContacts.h
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI.h"

@interface XingAPI (Contacts)

/**
 https://dev.xing.com/docs/get/users/:user_id/contacts

 Returns all contacts for the given user. This may make *many* requests, depending on the count
 of contacts the given user has. They are fetched in a given batch size (limit parameter). This
 means for a user with 1000 contacts it has to make 10 requests with a batch size of 100 to fetch
 them all.

 @param userID The userID (user_id) string the contacts should get fetched for.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingObjectContacts in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadAllContactsForUser:(NSString *)userID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadAllContactsWithParameters:(NSDictionary *)parameters;
- (RACSignal *)loadAllContactsForUser:(NSString *)userID;
- (RACSignal *)loadAllContacts;

/**
 https://dev.xing.com/docs/get/users/:user_id/contacts

 Returns the requested user's contacts.

 @param userID The userID (user_id) string the contacts should get fetched for.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingObjectContacts in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadContactsForUser:(NSString *)userID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadContactsWithParameters:(NSDictionary *)parameters;
- (RACSignal *)loadContactsForUser:(NSString *)userID;
- (RACSignal *)loadContacts;

/**
 https://dev.xing.com/docs/get/users/:user_id/contacts/:contact_id/tags

 TODO: Not implemented yet.
 */

/**
 https://dev.xing.com/docs/get/users/:user_id/contacts/shared

 Returns the list of contacts who are direct contacts of both the given and the currently
 authenticated user.

 @param userID The userID (user_id) string of a direct contact of the currently authenticated
 user. This cannot be `nil` and will not be replaced with `me`.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingObjectContacts in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadSharedContactsWithUser:(NSString *)userID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadSharedContactsWithUser:(NSString *)userID;

@end
