//
//  XingAPIContacts.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+Contacts.h"

@implementation XingAPI (Contacts)

#pragma mark - Public methods

- (RACSignal *)loadAllContactsWithParameters:(NSDictionary *)inParameters {
    return [self loadAllContactsForUser:nil parameters:inParameters];
}

- (RACSignal *)loadAllContactsForUser:(NSString *)userID {
    return [self loadAllContactsForUser:userID parameters:nil];
}

- (RACSignal *)loadAllContacts {
    return [self loadAllContactsForUser:nil parameters:nil];
}

- (RACSignal *)loadAllContactsForUser:(NSString *)userID parameters:(NSDictionary *)inParameters {
    RACReplaySubject *subject = [RACReplaySubject subject];
    NSMutableDictionary *parameters = [inParameters mutableCopy];

    parameters[@"limit"] = @0;
    __weak __typeof(&*self)weakSelf = self;

    [[self loadContactsForUser:userID parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        XingObjectContacts *contacts = response.resultObject;
        NSMutableArray *subjects = [NSMutableArray array];
        parameters[@"limit"] = inParameters[@"limit"];
        int limit = [inParameters[@"limit"] intValue];
        int total = [contacts.total intValue];
        [weakSelf withLimit:limit andTotal:total callBlock:^(int offset) {
            parameters[@"offset"] = @(offset);
            [subjects addObject:[self loadContactsForUser:userID parameters:parameters]];
        }];
        [[RACSignal merge:subjects] subscribeNext:^(XingAPIResponse *response) {
            [subject sendNext:response];
        } error:^(NSError *error) {
            [subject sendError:error];
        } completed:^{
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];

    return subject;
}

- (RACSignal *)loadContactsWithParameters:(NSDictionary *)parameters {
    return [self loadContactsForUser:nil parameters:parameters];
}

- (RACSignal *)loadContactsForUser:(NSString *)userID {
    return [self loadContactsForUser:userID parameters:nil];
}

- (RACSignal *)loadContacts {
    return [self loadContactsForUser:nil parameters:nil];
}

- (RACSignal *)loadContactsForUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/contacts", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingObjectContacts *contacts = (XingObjectContacts *)response.resultObject;
            [user addContacts:contacts.items];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadSharedContactsWithUser:(NSString *)userID {
    return [self loadSharedContactsWithUser:userID parameters:nil];
}

- (RACSignal *)loadSharedContactsWithUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    NSParameterAssert(userID);
    NSString *path = [NSString stringWithFormat:@"users/%@/contacts/shared", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingObjectContacts *contacts = (XingObjectContacts *)response.resultObject;
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:self.session.xingUserID inContext:context];
            XingEntityUser *contact = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            [user addContacts:contacts.items];
            [contact addContacts:contacts.items];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

#pragma mark - Private methods

@end
