//
//  XingAPIMessages.h
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI.h"

@interface XingAPI (Messages)

/**
 https://dev.xing.com/docs/put/users/:user_id/conversations/:conversation_id/participants/:id

 Add a contact of yours to a conversation.

 @param contactID The contactID (id) the contact's userID.
 @param conversationID The conversationID (conversation_id) the contact should be added to.
 */
- (RACSignal *)addContact:(NSString *)contactID toConversation:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/post/users/:user_id/conversations

 Starts a conversation by sending the passed message to the recipients. The subject of a
 conversation cannot be changed afterwards. Basic members are not allowed to send messages to
 non-contacts. Premium members are limited to 20 messages to non-contacts per month.

 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingObjectConversations in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)createConversation:(NSDictionary *)parameters;

/**
 https://dev.xing.com/docs/post/users/:user_id/conversations/:conversation_id/messages

 Creates a new message in an existing conversation. Premium members are limited to 20 messages to
 non-contacts per month.

 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @param conversationID The conversationID (conversation_id) the message should be created in.
 */
- (RACSignal *)createMessage:(NSDictionary *)parameters inConversation:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/delete/users/:user_id/conversations/:id

 Delete a conversation

 @param conversationID The conversationID (id) of the conversation that should be deleted.
 */
- (RACSignal *)deleteConversation:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/get/users/:user_id/conversations/:id

 Returns a single conversation.

 @param conversationID The conversationID (id) string of the conversation that should get fetched.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingEntityConversation in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadConversation:(NSString *)conversationID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadConversation:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/get/users/:user_id/conversations

 Returns the list of conversations for the given user.

 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingObjectConversations in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadConversations:(NSDictionary *)parameters;
- (RACSignal *)loadConversations;

/**
 https://dev.xing.com/docs/get/users/:user_id/conversations/:conversation_id/messages/:id

 Returns a single message.

 @param messageID The messageID (id) string of the message that should get fetched.
 @param conversationID The conversationID (conversation_id) string of the conversation the message
 belongs to.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingEntityMessage in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadMessage:(NSString *)messageID conversation:(NSString *)conversationID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadMessage:(NSString *)messageID conversation:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/get/users/:user_id/conversations/:conversation_id/messages

 Returns the messages for a conversation.

 @param conversationID The conversationID (id) string of the conversation that should get fetched.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingObjectMessages in `XingAPIResponse.resultObject` in the sendNext callback
 */
- (RACSignal *)loadMessages:(NSString *)conversationID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadMessages:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/put/users/:user_id/conversations/:id/read

 Marks all messages in the conversation as read.

 @param conversationID The conversationID (id) all messages should be marked as read in.
 */
- (RACSignal *)markAllMessagesAsRead:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/put/users/:user_id/conversations/:conversation_id/messages/:id/read

 Marks a message in a conversation as read.

 @param messageID The messageID (id) that should be marked as read.
 @param conversationID The conversationID (conversation_id) the message belongs to.
*/
- (RACSignal *)markMessageAsRead:(NSString *)messageID inConversation:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/delete/users/:user_id/conversations/:conversation_id/messages/:id/read

 Marks a message in a conversation as unread.

 @param messageID The messageID (id) that should be marked as read.
 @param conversationID The conversationID (conversation_id) the message belongs to.
 */
- (RACSignal *)markMessageAsUnread:(NSString *)messageID inConversation:(NSString *)conversationID;

/**
 https://dev.xing.com/docs/get/users/me/conversations/valid_recipients/:id

 TODO: Not implemented yet.

 @param userID The userID (id) of the recipient that should be validated.
 */
- (RACSignal *)validateRecipient:(NSString *)userID;

@end
