//
//  XingAPINetworkFeed.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+NetworkFeed.h"

@implementation XingAPI (NetworkFeed)

#pragma mark - Public methods

- (RACSignal *)createComment:(NSDictionary *)parameters inActivity:(NSString *)activityID {
    NSParameterAssert(activityID);
    NSParameterAssert(parameters);
    NSParameterAssert(parameters[XingAPIParamText]);
    NSString *path = [NSString stringWithFormat:@"activities/%@/comments", activityID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self postRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)createLikeOfActivity:(NSString *)activityID {
    NSParameterAssert(activityID);
    NSString *path = [NSString stringWithFormat:@"activities/%@/like", activityID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self putRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)deleteActivity:(NSString *)activityID {
    NSParameterAssert(activityID);
    NSString *path = [NSString stringWithFormat:@"activities/%@", activityID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self deleteRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)deleteComment:(NSString *)commentID inActivity:(NSString *)activityID {
    NSParameterAssert(activityID);
    NSParameterAssert(commentID);
    NSString *path = [NSString stringWithFormat:@"activities/%@/comments/%@", activityID, commentID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self deleteRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)deleteLikeOfActivity:(NSString *)activityID {
    NSParameterAssert(activityID);
    NSString *path = [NSString stringWithFormat:@"activities/%@/like", activityID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self deleteRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadActivity:(NSString *)activityID {
    return [self loadActivity:activityID parameters:nil];
}

- (RACSignal *)loadActivityComments:(NSString *)activityID {
    return [self loadActivityComments:activityID parameters:nil];
}

- (RACSignal *)loadActivityComments:(NSString *)activityID parameters:(NSDictionary *)parameters {
    NSParameterAssert(activityID);
    NSString *path = [NSString stringWithFormat:@"activities/%@/comments", activityID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            NSSet *result = [NSSet setWithArray:response.result.array];
            XingEntityActivity *activity = [XingEntityActivity MR_findFirstByAttribute:@"activityID" withValue:activityID inContext:context];

            if (activity) {
                [activity addComments:result];

                NSError *error = nil;
                [context save:&error];
                if (error) {
                    [subject sendError:error];
                }
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadActivityLikes:(NSString *)activityID {
    return [self loadActivityLikes:activityID parameters:nil];
}

- (RACSignal *)loadActivityLikes:(NSString *)activityID parameters:(NSDictionary *)parameters {
    NSParameterAssert(activityID);
    NSString *path = [NSString stringWithFormat:@"activities/%@/likes", activityID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            NSSet *result = [NSSet setWithArray:response.result.array];
            XingEntityActivity *activity = [XingEntityActivity MR_findFirstByAttribute:@"activityID" withValue:activityID inContext:context];

            if (activity) {
                [activity addLikes:result];

                NSError *error = nil;
                [context save:&error];
                if (error) {
                    [subject sendError:error];
                }
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadNetworkFeed {
    return [self loadNetworkFeedForUser:nil parameters:nil];
}

- (RACSignal *)loadNetworkFeed:(NSDictionary *)parameters {
    return [self loadNetworkFeedForUser:nil parameters:parameters];
}

- (RACSignal *)loadUserFeedWithParameters:(NSDictionary *)parameters {
    return [self loadUserFeedForUser:nil parameters:parameters];
}

- (RACSignal *)loadUserFeedForUser:(NSString *)userID {
    return [self loadUserFeedForUser:userID parameters:nil];
}

- (RACSignal *)loadUserFeed {
    return [self loadUserFeedForUser:nil parameters:nil];
}

- (RACSignal *)loadUserFeedForUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/feed", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            NSSet *result = [NSSet setWithArray:response.result.array];
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            [user addOwnActivities:result];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)shareActivity:(NSString *)activityID {
    return [self shareActivity:activityID parameters:nil];
}

- (RACSignal *)shareActivity:(NSString *)activityID parameters:(NSDictionary *)parameters {
    NSParameterAssert(activityID);
    NSString *path = [NSString stringWithFormat:@"activities/%@/share", activityID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self postRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

#pragma mark - Private methods

- (RACSignal *)loadActivity:(NSString *)activityID parameters:(NSDictionary *)parameters {
    NSParameterAssert(activityID);
    NSString *path = [NSString stringWithFormat:@"activities/%@", activityID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadNetworkFeedForUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/network_feed", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            NSSet *result = [NSSet setWithArray:response.result.array];
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            [user addNetworkActivities:result];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

@end
