//
//  XingAPIProfileMessages.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+ProfileMessages.h"

@implementation XingAPI (ProfileMessages)

#pragma mark - Public methods

- (RACSignal *)loadProfileMessage {
    return [self loadProfileMessageForUser:nil];
}

- (RACSignal *)loadProfileMessageForUser:(NSString *)userID {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/profile_message", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            XingEntityProfileMessage *message = (XingEntityProfileMessage *)response.resultObject;
            user.profileMessage = message;

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)updateProfileMessage:(NSDictionary *)parameters {
    NSParameterAssert(parameters);
    NSParameterAssert(parameters[XingAPIParamMessage]);
    NSString *path = [NSString stringWithFormat:@"users/%@/profile_message", self.session.xingUserID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self postRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

#pragma mark - Private methods

@end
