//
//  XingAPIProfileVisits.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+ProfileVisits.h"

@implementation XingAPI (ProfileVisits)

#pragma mark - Public methods

- (RACSignal *)createProfileVisit:(NSString *)visitedUserID {
    NSParameterAssert(visitedUserID);
    NSString *path = [NSString stringWithFormat:@"users/%@/visits", visitedUserID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self postRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadProfileVisits:(NSDictionary *)parameters {
    return [self loadProfileVisitsForUser:nil parameters:parameters];
}

- (RACSignal *)loadProfileVisits {
    return [self loadProfileVisitsForUser:nil parameters:nil];
}

#pragma mark - Private methods

- (RACSignal *)loadProfileVisitsForUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/visits", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            NSSet *result = [NSSet setWithArray:response.result.array];
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];

            NSMutableSet *profileVisits = [NSMutableSet set];
            for (XingEntityProfileVisit *visit in result) {
                [profileVisits addObject:visit.visitedAtEncrypted];
            }

            NSMutableSet *toDelete = [NSMutableSet set];
            for (XingEntityProfileVisit *visit in user.profileVisits) {
                if ([profileVisits containsObject:visit.visitedAtEncrypted]) {
                    [toDelete addObject:visit];
                }
            }

            for (XingEntityProfileVisit *visit in toDelete) {
                [user removeProfileVisitsObject:visit];
                [context deleteObject:visit];
            }

            [user addProfileVisits:result];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

@end
