//
//  XingAPIRecommendations.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+Recommendations.h"

@implementation XingAPI (Recommendations)

#pragma mark - Public methods

- (RACSignal *)blockUserRecommendation:(NSString *)userID {
    NSParameterAssert(userID);
    NSString *path = [NSString stringWithFormat:@"users/%@/network/recommendations/user/%@", self.session.xingUserID, userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self deleteRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:self.session.xingUserID inContext:context];
            XingEntityUser *recommendation = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];

            if (user != nil && recommendation != nil) {
                [user removeUserRecommendationsObject:recommendation];

                NSError *error = nil;
                [context save:&error];
                if (error) {
                    [subject sendError:error];
                }
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadJobRecommendations {
    return [self loadJobRecommendationsForUser:nil parameters:nil];
}

- (RACSignal *)loadJobRecommendations:(NSDictionary *)parameters {
    return [self loadJobRecommendationsForUser:nil parameters:parameters];
}

- (RACSignal *)loadUserRecommendations {
    return [self loadUserRecommendationsForUser:nil parameters:nil];
}

- (RACSignal *)loadUserRecommendations:(NSDictionary *)parameters {
    return [self loadUserRecommendationsForUser:nil parameters:parameters];
}

#pragma mark - Private methods

- (RACSignal *)loadUserRecommendationsForUser:(NSString *)userID
                                   parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/network/recommendations", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingObjectUserRecommendations *recos = (XingObjectUserRecommendations *)response.resultObject;
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            [user addUserRecommendations:recos.items];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadJobRecommendationsForUser:(NSString *)userID
                                  parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@/jobs/recommendations", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        NSManagedObjectContext *context = response.operation.managedObjectContext;
        [context performBlock:^{
            XingObjectJobRecommendations *recos = (XingObjectJobRecommendations *)response.resultObject;
            XingEntityUser *user = [XingEntityUser MR_findFirstByAttribute:@"userID" withValue:userID inContext:context];
            [user addJobRecommendations:recos.items];

            NSError *error = nil;
            [context save:&error];
            if (error) {
                [subject sendError:error];
            }

            [subject sendNext:response];
            [subject sendCompleted];
        }];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

@end
