//
//  XingAPIUserProfiles.h
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI.h"

@interface XingAPI (UserProfiles)

/**
 https://dev.xing.com/docs/get/users/find_by_emails

 Returns the list of users that belong directly to the given list of email addresses. At least the
 `emails` key in the `parameter` dictionary must be set.

 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingObjectFindByEmails in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)findByEmails:(NSDictionary *)parameters;

/**
 https://dev.xing.com/docs/get/users/me/id_card

 Returns minimal profile information of the user that authorized the consumer.

 @return XingEntityUser in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadIDCard;

/**
 https://dev.xing.com/docs/get/users/:id
 https://dev.xing.com/docs/get/users/me

 Shows a particular user's profile. The data returned by this call will be checked against and
 filtered on the basis of the privacy settings of the requested user.

 @param userID The userID (id) string of the user that should get fetched.
 @param parameters A dictionary of GET parameters. See the documentation linked above for valid
 parameters and their meaning.
 @return XingEntityUser in `XingAPIResponse.resultObject` in the `sendNext` callback.
 */
- (RACSignal *)loadUser:(NSString *)userID parameters:(NSDictionary *)parameters;
- (RACSignal *)loadUserWithParameters:(NSDictionary *)parameters;
- (RACSignal *)loadUser:(NSString *)userID;
- (RACSignal *)loadUser;

@end
