//
//  XingAPIUserProfiles.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/16/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingAPI+UserProfiles.h"

@implementation XingAPI (UserProfiles)

#pragma mark - Public methods

- (RACSignal *)findByEmails:(NSDictionary *)parameters {
    NSParameterAssert(parameters);
    NSParameterAssert(parameters[XingAPIParamEmails]);
    NSString *path = @"users/find_by_emails";
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadIDCard {
    NSString *path = @"users/me/id_card";
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:nil] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

- (RACSignal *)loadUserWithParameters:(NSDictionary *)parameters {
    return [self loadUser:nil parameters:parameters];
}

- (RACSignal *)loadUser:(NSString *)userID {
    return [self loadUser:userID parameters:nil];
}

- (RACSignal *)loadUser {
    return [self loadUser:nil parameters:nil];
}

- (RACSignal *)loadUser:(NSString *)userID parameters:(NSDictionary *)parameters {
    userID = userID ? userID : self.session.xingUserID;
    NSString *path = [NSString stringWithFormat:@"users/%@", userID];
    RACReplaySubject *subject = [RACReplaySubject subject];
    [[self getRequestAtPath:path parameters:parameters] subscribeNext:^(XingAPIResponse *response) {
        [subject sendNext:response];
        [subject sendCompleted];
    } error:^(NSError *error) {
        [subject sendError:error];
    }];
    return subject;
}

#pragma mark - Private methods

@end
