//
//  XingAPI.h
//  XingSDK
//
//  Created by Johannes Plunien on 12/1/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <NSManagedObject+MagicalFinders.h>
#import <ReactiveCocoa/ReactiveCocoa.h>
#import <RestKit/RestKit.h>
#import "SPDeepCopy.h"

#import "XingAPIConstants.h"
#import "XingAPIResponse.h"
#import "XingEntities.h"
#import "XingErrorMessage.h"
#import "XingObjectManager.h"
#import "XingSession.h"

/**
 The `XingAPI` class provides access to all API resources. They are put into groups in a similar
 way as in the API documentation: https://dev.xing.com/docs/resources

 */
@interface XingAPI : NSObject

@property (nonatomic, readonly) XingObjectManager *manager;
@property (nonatomic, readonly) XingSession *session;
@property (nonatomic, readonly) RACSubject *globalSignal;
@property (nonatomic, strong) NSManagedObjectContext *context;

- (id)initWithManager:(XingObjectManager *)inManager session:(XingSession *)inSession;
- (RACSignal *)deleteRequestAtPath:(NSString *)path parameters:(NSDictionary *)parameters;
- (RACSignal *)getRequestAtPath:(NSString *)path parameters:(NSDictionary *)parameters;
- (RACSignal *)postRequestAtPath:(NSString *)path parameters:(NSDictionary *)parameters;
- (RACSignal *)putRequestAtPath:(NSString *)path parameters:(NSDictionary *)parameters;
- (void)withLimit:(int)limit andTotal:(int)total callBlock:(void (^)(int offset))block;

@end
