//
//  XingEntityJob.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/21/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingEntityJob.h"
#import "XingEntityCompany.h"
#import "XingEntityLocation.h"
#import "XingEntityUser.h"

@implementation XingEntityJob

+ (RKEntityMapping *)mappingInManagedObjectStore:(RKManagedObjectStore *)store {
    RKEntityMapping *mapping = [RKEntityMapping mappingForEntityForName:[self entityName]
                                                   inManagedObjectStore:store];

    mapping.identificationAttributes = @[@"jobID"];

    [mapping addAttributeMappingsFromDictionary:@{
     @"industry": @"industry",
     @"description": @"jobDescription",
     @"id": @"jobID",
     @"job_type": @"jobType",
     @"level": @"level",
     @"links": @"links",
     @"published_at": @"publishedAt",
     @"skills": @"skills",
     @"tags": @"tags",
     @"title": @"title",
     }];

    RKRelationshipMapping *relationshipMapping = nil;
    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"location"
                                                                      toKeyPath:@"location"
                                                                    withMapping:[XingEntityLocation mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"company"
                                                                      toKeyPath:@"company"
                                                                    withMapping:[XingEntityCompany mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"contact.company"
                                                                      toKeyPath:@"contactCompany"
                                                                    withMapping:[XingEntityCompany mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"contact.user"
                                                                      toKeyPath:@"contactUser"
                                                                    withMapping:[XingEntityUser mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    return mapping;
}

- (NSString *)description {
    return [NSString stringWithFormat:@"<%@:%p title:%@>", NSStringFromClass([self class]), self, self.title];
}

@end
