//
//  XingUser.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/1/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingEntityUser.h"
#import "RKResponseMapperOperation.h"

@implementation XingEntityUser

+ (RKEntityMapping *)mappingInManagedObjectStore:(RKManagedObjectStore *)store {
    RKEntityMapping *mapping = [RKEntityMapping mappingForEntityForName:[self entityName]
                                                   inManagedObjectStore:store];
    mapping.identificationAttributes = @[@"userID"];

    [mapping addAttributeMappingsFromDictionary:@{
     @"id":                             @"userID",
     @"first_name":                     @"firstName",
     @"last_name":                      @"lastName",
     @"display_name":                   @"displayName",
     @"page_name":                      @"pageName",
     @"permalink":                      @"permalink",
     @"active_email":                   @"activeEmail",
     @"gender":                         @"gender",
     @"wants":                          @"wants",
     @"haves":                          @"haves",
     @"interests":                      @"interests",
     @"organisation_member":            @"organisationMember",
     @"premium_services":               @"premiumServices",
     @"badges":                         @"badges",
     @"languages":                      @"languages",
     @"web_profiles":                   @"webProfiles",
     @"instant_messaging_accounts":     @"instantMessagingAccounts",
     }];

    RKRelationshipMapping *relationshipMapping = nil;
    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"birth_date"
                                                                      toKeyPath:@"birthDate"
                                                                    withMapping:[XingEntityBirthDate mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"time_zone"
                                                                      toKeyPath:@"timeZone"
                                                                    withMapping:[XingEntityTimeZone mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"private_address"
                                                                      toKeyPath:@"privateAddress"
                                                                    withMapping:[XingEntityAddressPrivate mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"business_address"
                                                                      toKeyPath:@"businessAddress"
                                                                    withMapping:[XingEntityAddressBusiness mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"professional_experience"
                                                                      toKeyPath:@"professionalExperience"
                                                                    withMapping:[XingEntityProfessionalExperience mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"educational_background"
                                                                      toKeyPath:@"educationalBackground"
                                                                    withMapping:[XingEntityEducationalBackground mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    relationshipMapping = [RKRelationshipMapping relationshipMappingFromKeyPath:@"photo_urls"
                                                                      toKeyPath:@"photoURL"
                                                                    withMapping:[XingEntityPhotoURL mappingInManagedObjectStore:store]];
    relationshipMapping.assignmentPolicy = RKReplaceAssignmentPolicy;
    [mapping addPropertyMapping:relationshipMapping];

    return mapping;
}

#pragma mark - Instance methods

- (NSString *)description {
    return [NSString stringWithFormat:@"<%@:%p userID:%@ pageName:%@>", NSStringFromClass([self class]), self, self.userID, self.pageName];
}

- (BOOL)isContactOf:(XingEntityUser *)user {
    for (XingEntityUser *contact in user.contacts) {
        if ([contact.userID isEqualToString:self.userID]) {
            return YES;
        }
    }
    return NO;
}

@end
