//
//  XingOAuthAuthentication.m
//  XingSDK
//
//  Created by Johannes Plunien on 11/19/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingOAuthAuthentication.h"

@interface GTMOAuthAuthentication ()

- (void)setKeysForResponseDictionary:(NSDictionary *)dict;
- (NSMutableArray *)paramsForKeys:(NSArray *)keys request:(NSURLRequest *)request;
+ (NSString *)paramStringForParams:(NSArray *)params
                            joiner:(NSString *)joiner
                       shouldQuote:(BOOL)shouldQuote
                        shouldSort:(BOOL)shouldSort;

@end

@implementation GTMOAuthAuthentication (ExtendPublicInterface)

- (NSDictionary *)paramValues {
    return paramValues_;
}

@end

@implementation XingOAuthAuthentication

static NSString *const kOAuthTokenKey          = @"oauth_token";
static NSString *const kOAuthTokenSecretKey    = @"oauth_token_secret";
static NSString *const kServiceProviderKey     = @"serviceProvider";
static NSString *const kUserEmailKey           = @"email";
static NSString *const kUserEmailIsVerifiedKey = @"isVerified";
static NSString *const kXingUserIDKey          = @"user_id";

// this returns a "response string" that can be passed later to
// setKeysForResponseString: to reuse an old access token in a new auth object
- (NSString *)persistenceResponseString {
    // make an array of OAuthParameters for the actual parameters we're
    // persisting
    NSArray *persistenceKeys = [NSArray arrayWithObjects:
                                kOAuthTokenKey,
                                kOAuthTokenSecretKey,
                                kServiceProviderKey,
                                kXingUserIDKey,
                                kUserEmailKey,
                                kUserEmailIsVerifiedKey,
                                nil];

    NSMutableArray *params = [self paramsForKeys:persistenceKeys request:nil];
    NSString *responseStr = [[self class] paramStringForParams:params
                                                        joiner:@"&"
                                                   shouldQuote:NO
                                                    shouldSort:NO];
    return responseStr;
}

- (void)reset {
    [super reset];
    self.xingUserID = nil;
}

- (void)setKeysForResponseDictionary:(NSDictionary *)dict {
    [super setKeysForResponseDictionary:dict];
    NSString *userID = [dict objectForKey:kXingUserIDKey];
    if (userID) {
        self.xingUserID = userID;
    }
}

- (NSString *)xingUserID {
    return [[self paramValues] objectForKey:kXingUserIDKey];
}

- (void)setXingUserID:(NSString *)str {
    [[self paramValues] setValue:[str copy] forKey:kXingUserIDKey];
}

@end
