//
//  XingAPIRegressionTest.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/21/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingAPIRegressionTest.h"

@interface XingAPIRegressionTest ()

@property (nonatomic, strong) NSString *jobID;

@end

@implementation XingAPIRegressionTest

- (void)setUp {
    [super setUp];
}

// There were too many objects created. If we fetch the same user consecutively again and again,
// the count of objects should just stay the same.
- (void)testRegressionTooManyObjects {
    if (self.session.isReady == NO) return;

    // The user must be there before we can make other requests, so that the relationships get
    // connected properly.
    [self waitFor:[self.api loadUser:@"me"]];

    [self waitFor:[RACSignal merge:@[
                   [self.api loadContactRequests],
                   [self.api loadContacts],
                   [self.api loadConversations:@{@"with_latest_messages": @10}],
                   [self.api loadNetworkFeed],
                   [self.api loadProfileVisits],
                   ]]];

    XingAPIResponse *response = nil;
    [self waitFor:[self.api findJob:@{@"query":@"rails"}] result:&response];
    XingObjectJobs *jobs = response.resultObject;
    self.jobID = ((XingEntityJob *)jobs.items.anyObject).jobID;

    NSManagedObjectModel *model = XingSDK.manager.managedObjectStore.managedObjectModel;
    NSArray *entities = [[model entities] valueForKey:@"name"];

    NSMutableDictionary *before = [[NSMutableDictionary alloc] initWithCapacity:entities.count];
    for (NSString *entity in entities) {
        int count = (int)[NSClassFromString(entity) performSelector:@selector(MR_countOfEntities)];
        before[entity] = [NSString stringWithFormat:@"%@:%u", entity, count];
    }

    NSMutableArray *requests = [NSMutableArray array];
    for (int i=0; i<2; i++) {
        [requests addObjectsFromArray:@[
         [self.api loadContactRequests],
         [self.api loadContacts],
         [self.api loadConversations:@{@"with_latest_messages": @10}],
         [self.api loadJob:self.jobID],
         [self.api loadNetworkFeed],
         [self.api loadProfileVisits],
         [self.api loadUser:@"me"],
         ]];
    }
    [self waitFor:[RACSignal merge:requests]];

    for (NSString *entity in entities) {
        int count = (int)[NSClassFromString(entity) performSelector:@selector(MR_countOfEntities)];
        NSString *now = [NSString stringWithFormat:@"%@:%u", entity, count];
        expect(now).to.equal(before[entity]);
    }
}

@end
