//
//  XingActivityAPITest.m
//  XingSDK
//
//  Created by Johannes Plunien on 1/13/13.
//  Copyright (c) 2013 XING AG. All rights reserved.
//

#import "XingActivityAPITest.h"

@implementation XingActivityAPITest

- (void)testLoadNetworkFeedForUser {
    if (self.session.isReady == NO) return;

    // Reset the activities
    self.user.networkActivities = nil;
    [self.user.managedObjectContext save:nil];

    XingAPIResponse *response = nil;
    [self waitFor:self.api.loadNetworkFeed result:&response];
    NSArray *activities = response.result.array;

    expect(self.user.networkActivities.count).to.beGreaterThan(0);
    expect(self.user.networkActivities.count).to.equal(activities.count);
}

- (void)testLoadUserFeedForUser {
    if (self.session.isReady == NO) return;

    // Reset the activities
    self.user.networkActivities = nil;
    [self.user.managedObjectContext save:nil];

    XingAPIResponse *response = nil;
    [self waitFor:self.api.loadUserFeed result:&response];
    NSArray *activities = response.result.array;

    expect(self.user.ownActivities.count).to.beGreaterThan(0);
    expect(self.user.ownActivities.count).to.equal(activities.count);
}

- (void)testLoadActivity {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api loadActivity:@"8060_317a54"] result:&response];

    expect(response.resultObject).to.beInstanceOf(XingEntityActivity.class);

    XingEntityActivity *activity = response.resultObject;
    expect(activity.activityID).to.equal(@"8060_317a54");
}

- (void)testLoadActivityComments {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;

    // Make sure the activity is there
    [self waitFor:[self.api loadActivity:@"8060_317a54"] result:&response];

    [self waitFor:[self.api loadActivityComments:@"8060_317a54"] result:&response];

    // Fetch the activity again
    XingEntityActivity *activity = [XingEntityActivity MR_findFirstByAttribute:@"activityID" withValue:@"8060_317a54"];

    expect(response.result.array.lastObject).to.beInstanceOf(XingEntityActivityComment.class);
    expect(activity.comments.count).to.equal(response.result.array.count);
}

- (void)testLoadActivityLikes {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;

    // Make sure the activity is there
    [self waitFor:[self.api loadActivity:@"8060_317a54"] result:&response];

    [self waitFor:[self.api loadActivityLikes:@"8060_317a54"] result:&response];

    // Fetch the activity again
    XingEntityActivity *activity = [XingEntityActivity MR_findFirstByAttribute:@"activityID" withValue:@"8060_317a54"];

    expect(response.result.array.lastObject).to.beInstanceOf(XingEntityUser.class);
    expect(activity.likes.count).to.equal(response.result.array.count);
}

- (void)testShareActivity {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api shareActivity:@"8060_317a54"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testDeleteActivity {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api deleteActivity:@"8060_317a54"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testCreateCommentInActivity {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api createComment:@{XingAPIParamText: @"Comment!"} inActivity:@"8060_317a54"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(201);
}

- (void)testDeleteCommentInActivity {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api deleteComment:@"9050_333a54" inActivity:@"8060_317a54"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testCreateLikeFOfActivity {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api createLikeOfActivity:@"8060_317a54"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testDeleteLikeOfActivity {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api deleteLikeOfActivity:@"8060_317a54"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

@end
