//
//  XingContactAPITest.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/21/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingContactAPITest.h"

@implementation XingContactAPITest

- (void)testLoadContactsForUser {
    if (self.session.isReady == NO) return;

    // Reset the contacts
    self.user.contacts = nil;
    [self.user.managedObjectContext save:nil];

    XingAPIResponse *response = nil;
    [self waitFor:self.api.loadContacts result:&response];

    XingObjectContacts *contacts = response.resultObject;
    expect(contacts).to.beInstanceOf(XingObjectContacts.class);
    expect(contacts.total).to.beGreaterThan(0);
    expect(contacts.items.count).to.equal(self.user.contacts.count);

    for (XingEntityUser *user in contacts.items) {
        expect([user isContactOf:self.user]).to.equal(YES);
    }
}

- (void)testLoadAllContactsForUser {
    if (self.session.isReady == NO) return;

    // Reset the contacts
    self.user.contacts = nil;
    [self.user.managedObjectContext save:nil];

    XingAPIResponse *response = nil;
    [self waitFor:[self.api loadAllContactsWithParameters:@{@"limit": @100}] result:&response];

    XingObjectContacts *contacts = response.resultObject;
    expect(contacts.total.intValue).to.equal(self.user.contacts.count);
}

- (void)testLoadSharedContactsWithUser {
    if (self.session.isReady == NO) return;

    // Load some contacts for the current user
    XingAPIResponse *response = nil;
    [self waitFor:[self.api loadContactsWithParameters:@{@"limit": @1}] result:&response];

    // Define a user we will load the shared contacts for
    XingObjectContacts *contacts = response.resultObject;
    XingEntityUser *contact = (XingEntityUser *)contacts.items.anyObject;

    // Reset the contacts
    contact.contacts = nil;
    self.user.contacts = nil;
    [self.user.managedObjectContext save:nil];

    [self waitFor:[self.api loadSharedContactsWithUser:contact.userID] result:&response];
    XingObjectContacts *result = response.resultObject;

    for (XingEntityUser *user in result.items) {
        expect([user isContactOf:contact]).to.equal(YES);
        expect([user isContactOf:self.user]).to.equal(YES);
    }
}

@end
