//
//  XingMessageAPITest.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/21/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingMessageAPITest.h"

@implementation XingMessageAPITest

- (void)testLoadConversationsForUser {
    if (self.session.isReady == NO) return;

    // Reset the conversations
    self.user.conversations = nil;
    [self.user.managedObjectContext save:nil];

    XingAPIResponse *response = nil;
    [self waitFor:self.api.loadConversations result:&response];
    XingObjectConversations *conversations = response.resultObject;

    expect(conversations).to.beInstanceOf(XingObjectConversations.class);
    expect(conversations.total).to.beGreaterThan(0);
    expect(conversations.items.count).to.equal(self.user.conversations.count);
}

- (void)testLoadConversationForUser {
    if (self.session.isReady == NO) return;

    // Load some conversations, we just need any conversationID
    XingAPIResponse *response = nil;
    [self waitFor:self.api.loadConversations result:&response];
    XingObjectConversations *conversations = response.resultObject;
    NSString *conversationID = ((XingEntityConversation *)conversations.items.anyObject).conversationID;

    // Reset the conversations
    self.user.conversations = nil;
    [self.user.managedObjectContext save:nil];

    // Load one conversation
    [self waitFor:[self.api loadConversation:conversationID] result:&response];
    XingEntityConversation *conversation = response.resultObject;

    expect(conversation).to.beInstanceOf(XingEntityConversation.class);
    expect(self.user.conversations.count).to.equal(1);
    expect(self.user.conversations.anyObject).to.equal(conversation);
}

- (void)testCreateConversationForUser {
    if (self.session.isReady == NO) return;

    // Reset the conversations
    self.user.conversations = nil;
    [self.user.managedObjectContext save:nil];

    NSDictionary *params = @{
    XingAPIParamContent : @"Body",
    XingAPIParamRecipientIDS : @"18600191_ea2e1c",
    XingAPIParamSubject : @"Subject",
    };

    XingAPIResponse *response = nil;
    [self waitFor:[self.api createConversation:params] result:&response];
    XingEntityConversation *conversation = response.resultObject;

    expect(conversation).to.beInstanceOf(XingEntityConversation.class);
    expect(self.user.conversations.count).to.equal(1);
    expect(self.user.conversations.anyObject).to.equal(conversation);
}

- (void)testMarkAllMessagesAsReadInConversation {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api markAllMessagesAsRead:@"467437519_1714c5"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testAddParticipantToConversation {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api addContact:@"18600191_ea2e1c" toConversation:@"467437519_1714c5"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testMarkMessageAsRead {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api markMessageAsRead:@"18600191_ea2e1c" inConversation:@"467437519_1714c5"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testMarkMessageAsUnread {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api markMessageAsUnread:@"18600191_ea2e1c" inConversation:@"467437519_1714c5"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testLoadMessagesForConversation {
    if (self.session.isReady == NO) return;

    // Load some conversations, we just need any conversationID
    XingAPIResponse *response = nil;
    [self waitFor:self.api.loadConversations result:&response];
    XingObjectConversations *conversations = response.resultObject;
    XingEntityConversation *conversation = (XingEntityConversation *)conversations.items.anyObject;

    // Reset the conversations
    self.user.conversations = nil;
    [self.user.managedObjectContext save:nil];

    // Reset the messages
    conversation.messages = nil;
    [conversation.managedObjectContext save:nil];

    // Load messages
    [self waitFor:[self.api loadMessages:conversation.conversationID] result:&response];
    XingObjectMessages *messages = response.resultObject;

    expect(messages).to.beInstanceOf(XingObjectMessages.class);
    expect(messages.total).to.beGreaterThan(0);
    expect(self.user.conversations.count).to.equal(1);
    expect(conversation.messages.count).to.equal(messages.items.count);
}

- (void)testLoadMessage {
    if (self.session.isReady == NO) return;

    // Load some conversations, we just need any conversationID and any messageID
    XingAPIResponse *response = nil;
    [self waitFor:[self.api loadConversations:@{@"with_latest_messages": @10}]
           result:&response];
    XingObjectConversations *conversations = response.resultObject;
    XingEntityConversation *conversation = (XingEntityConversation *)conversations.items.anyObject;

    // TODO: <plu> something's wrong here. Why can we only fetch a message where the authenticated
    // user is the sender? This does not make sense. If the conversation is readable by the
    // authenticated user, so should be all messages of that conversation.
    XingEntityMessage *message = nil;
    for (XingEntityMessage *item in conversation.messages) {
        if ([item.sender.userID isEqualToString:self.session.xingUserID]) {
            message = item;
            break;
        }
    }

    // Reset the conversations
    self.user.conversations = nil;
    [self.user.managedObjectContext save:nil];

    // Reset the messages
    conversation.messages = nil;
    [conversation.managedObjectContext save:nil];

    // Load the message
    [self waitFor:[self.api loadMessage:message.messageID conversation:conversation.conversationID]
           result:&response];
    XingEntityMessage *result = response.resultObject;

    expect(result).to.beInstanceOf(XingEntityMessage.class);
    expect(result.messageID).to.equal(message.messageID);
    expect(conversation.messages.count).to.equal(1);
    expect(conversation.messages.anyObject).to.equal(result);
    expect(self.user.conversations.count).to.equal(1);
}

- (void)testCreateMessageInConversation {
    if (self.session.isReady == NO) return;

    NSDictionary *params = @{
    XingAPIParamContent : @"Body",
    };

    XingAPIResponse *response = nil;
    [self waitFor:[self.api createMessage:params inConversation:@"654321_ghijkl"] result:&response];
    XingEntityMessage *message = response.resultObject;

    expect(message).to.beInstanceOf(XingEntityMessage.class);
}

- (void)testDeleteConversation {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api deleteConversation:@"467437519_1714c5"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

@end
