//
//  XingUserAPITest.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/21/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingUserAPITest.h"

@implementation XingUserAPITest

- (void)testLoadUser {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:self.api.loadUser result:&response];
    XingEntityUser *user = response.resultObject;

    expect(user).to.beInstanceOf(XingEntityUser.class);
    expect(user.userID).to.equal(self.session.xingUserID);
    expect(user.managedObjectContext).notTo.beNil();
}

- (void)testLoadMeUser {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api loadUser:@"me"] result:&response];
    XingEntityUser *user = response.resultObject;

    expect(user).to.beInstanceOf(XingEntityUser.class);
    expect(user.userID).to.equal(self.session.xingUserID);
    expect(user.managedObjectContext).notTo.beNil();
}

- (void)testLoadIDCard {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api loadIDCard] result:&response];
    XingEntityUser *user = response.resultObject;

    expect(user).to.beInstanceOf(XingEntityUser.class);
    expect(user.userID).to.equal(self.session.xingUserID);
    expect(user.managedObjectContext).notTo.beNil();
}

- (void)testLoadUserRecommendations {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:self.api.loadUserRecommendations result:&response];
    XingObjectUserRecommendations *recos = response.resultObject;

    expect(recos).to.beInstanceOf(XingObjectUserRecommendations.class);
    expect(recos.total).to.beGreaterThan(0);
    expect(recos.items.anyObject).to.beInstanceOf(XingEntityUser.class);
    expect(self.user.userRecommendations.count).to.equal(recos.items.count);
}

- (void)testBlockUserRecommendation {
    if (self.session.isReady == NO) return;

    XingAPIResponse *response = nil;
    [self waitFor:[self.api blockUserRecommendation:@"467437519_1714c5"] result:&response];
    expect(response.operation.HTTPRequestOperation.response.statusCode).to.equal(204);
}

- (void)testFindByEmails {
    if (self.session.isReady == NO) return;

    NSString *email = @"plu@pqpq.de";
    XingAPIResponse *response = nil;
    [self waitFor:[self.api findByEmails:@{@"emails":email}] result:&response];
    XingObjectFindByEmails *result = response.resultObject;

    XingObjectFindByEmailsItem *item = (XingObjectFindByEmailsItem *)result.items.anyObject;

    expect(result).to.beInstanceOf(XingObjectFindByEmails.class);
    expect(result.total).to.beGreaterThan(0);
    expect(item).to.beInstanceOf(XingObjectFindByEmailsItem.class);
    expect(item.email).to.equal(email);
    expect(item.user).to.beInstanceOf(XingEntityUser.class);
}

@end
