//
//  XingAPITest.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/14/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingAPITest.h"

@interface XingAPITest ()

@end

@implementation XingAPITest

- (void)setUp {
    [super setUp];
    [self setupSDK];
    self.timeout = 5.0f;
}

- (void)setupSDK {
    NSBundle *bundle = [NSBundle bundleForClass:[self class]];
    NSString *filePath = [bundle pathForResource:@"XingSDKTestConfig" ofType:@"json"];

    NSError *error = nil;
    NSData *json = [NSData dataWithContentsOfFile:filePath];
    NSDictionary *config = [NSJSONSerialization JSONObjectWithData:json
                                                           options:NSJSONReadingMutableContainers
                                                             error:&error];

    NSAssert(error == nil, @"Could not parse test config from file");

    self.sdk = [[XingSDK alloc] init];

    if (config[@"apiURL"] != nil) {
        self.sdk = [[XingSDK alloc] initWithURL:config[@"apiURL"]];
    }

    if (config[@"consumerKey"] != nil) {
        self.session.consumerKey = config[@"consumerKey"];
    }

    if (config[@"consumerSecret"] != nil) {
        self.session.consumerSecret = config[@"consumerSecret"];
    }

    if (config[@"accessToken"] != nil) {
        self.session.accessToken = config[@"accessToken"];
    }

    if (config[@"accessTokenSecret"] != nil) {
        self.session.accessTokenSecret = config[@"accessTokenSecret"];
    }

    if (config[@"xingUserID"] != nil) {
        self.session.xingUserID = config[@"xingUserID"];
    }

    if (config[@"callback"] != nil) {
        self.session.callback = config[@"callback"];
    }
}

#pragma mark - Public methods

- (XingAPI *)api {
    return self.sdk.api;
}

- (XingSession *)session {
    return self.sdk.session;
}

- (XingEntityUser *)user {
    if (_user == nil) {
        XingAPIResponse *response = nil;
        [self waitFor:self.api.loadUser result:&response timeout:self.timeout];
        _user = (XingEntityUser *)response.resultObject;
    }
    return _user;
}

- (void)waitFor:(RACSignal *)subject result:(NSObject **)result {
    [self waitFor:subject result:result timeout:self.timeout];
}

- (void)waitFor:(RACSignal *)subject {
    [self waitFor:subject timeout:self.timeout];
}

- (void)waitFor:(RACSignal *)subject result:(NSObject **)result timeout:(float)timeout {
    __block id next;
    [subject subscribeNext:^(id x) {
        next = x;
    } error:^(NSError *error) {
        NSLog(@"ERROR loading result: %@", error);
    }];
    [self waitFor:subject timeout:timeout];
    *result = next;
}

- (void)waitFor:(RACSignal *)subject timeout:(float)timeout {
    NSDate *timeoutDate = [NSDate dateWithTimeIntervalSinceNow:timeout];
    __block BOOL done = NO;
    [subject subscribeError:^(NSError *error) {
        done = YES;
    } completed:^{
        done = YES;
    }];
    do {
        NSDate *stopDate = [NSDate dateWithTimeIntervalSinceNow:0.001];
        [[NSRunLoop currentRunLoop] runUntilDate:stopDate];
        NSAssert([timeoutDate timeIntervalSinceNow] > 0.0, @"Timed out");
    } while (!done);
}

@end
