//
//  XingActivityMappingTest.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/22/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "RKISO8601DateFormatter.h"
#import "XingActivityMappingTest.h"

@implementation XingActivityMappingTest

- (void)testActivityMapping {
    RKEntityMapping *mapping = [self mappingForClass:[XingEntityActivity class]];
    NSDictionary *data = [self mappingData:@"activity"][@"network_activities"][0];
    RKMappingTest *test = [self testForMapping:mapping withData:data];
    [test performMapping];

    NSArray *changes = @[@"haves"];
    NSDate *createdAt = [[[RKISO8601DateFormatter alloc] init] dateFromString:@"2011-05-26T11:14:38Z"];
    NSArray *possibleActions = @[@"COMMENT", @"LIKE"];

    XingEntityActivity *activity = test.destinationObject;
    expect(activity.aggregated).to.equal(@NO);
    expect(activity.aggregatedBy).to.equal(@"key");
    expect(activity.changes).to.equal(changes);
    expect(activity.createdAt).to.equal(createdAt);
    expect(activity.possibleActions).to.equal(possibleActions);
    expect(activity.type).to.equal(@"activity");
    expect(activity.verb).to.equal(@"update");
    expect(activity.totalComments).to.equal(@1);
    expect(activity.currentUserCommented).to.equal(@YES);
    expect(activity.totalLikes).to.equal(@32);
    expect(activity.currentUserLiked).to.equal(@NO);

    expect(activity.objects.count).to.equal(((NSArray *)data[@"objects"]).count);

    XingEntityActivityObject *object = activity.objects[0];
    expect(object.objectType).to.equal(@"user");
    expect(object.objectData[@"id"]).to.equal(@"6666666_6a0d84");
    expect(object.objectData[@"type"]).to.equal(@"user");

    XingEntityActivityActor *actor = activity.actors[0];
    expect(actor.objectType).to.equal(@"company");
    expect(actor.objectData[@"id"]).to.equal(@"6655566_6a0d84");
    expect(actor.objectData[@"type"]).to.equal(@"company");

    NSDate *commentCreatedAt = [[[RKISO8601DateFormatter alloc] init] dateFromString:@"2013-01-11T17:14:35Z"];
    expect(activity.comments.anyObject).to.beInstanceOf(XingEntityActivityComment.class);
    XingEntityActivityComment *comment = activity.comments.anyObject;
    expect(comment.content).to.equal(@"Comment content ...");
    expect(comment.commentID).to.equal(@"290760388_781a4d");
    expect(comment.createdAt).to.equal(commentCreatedAt);
    expect(comment.creator).to.beInstanceOf(XingEntityUser.class);
}

@end
