//
//  XingJobMappingTest.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/21/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingJobMappingTest.h"
#import "RKISO8601DateFormatter.h"

@implementation XingJobMappingTest

- (void)testJobMapping {
    RKEntityMapping *mapping = [self mappingForClass:[XingEntityJob class]];
    NSDictionary *data = [self mappingData:@"job"][@"job"];
    RKMappingTest *test = [self testForMapping:mapping withData:data];
    [test performMapping];

    NSArray *skills = @[@"API design", @"Ruby", @"Foosball", @"Kicker", @"Rails"];
    NSArray *tags = @[@"computer", @"it"];
    NSDate *publishedAt = [[[RKISO8601DateFormatter alloc] init] dateFromString:@"2012-08-06 13:14:48 +0000"];
    NSDictionary *links = @{
    @"pdf": @"https://www.xing.com/jobs/foo.pdf",
    @"self": @"https://api.xing.com/v1/jobs/61723_4cae01",
    @"xing": @"http://www.xing.com/jobs/hamburg-programmierer-13",
    };

    XingEntityJob *job = test.destinationObject;
    expect(job.jobID).to.equal(@"61723_4cae01");
    expect(job.title).to.equal(@"Programmierer");
    expect(job.level).to.equal(@"PROFESSIONAL_EXPERIENCED");
    expect(job.jobType).to.equal(@"FULL_TIME");
    expect(job.industry).to.equal(@"INTERNET");
    expect(job.skills).to.equal(skills);
    expect(job.jobDescription).to.equal(@"Some Job Description.");
    expect(job.tags).to.equal(tags);
    expect(job.publishedAt).to.equal(publishedAt);
    expect(job.links).to.equal(links);
    expect(job.contactCompany).to.beInstanceOf(XingEntityCompany.class);
    expect(job.contactUser).to.beInstanceOf(XingEntityUser.class);

    expect(job.location).to.beInstanceOf(XingEntityLocation.class);
    XingEntityLocation *location = job.location;
    expect(location.city).to.equal(@"Hamburg");
    expect(location.country).to.equal(@"DE");
    expect(location.street).to.equal(@"Gänsemarkt 43");
    expect(location.zipCode).to.equal(@"20354");
    expect(location.region).to.equal(@"Hamburg");

    expect(location.geoCode).to.beInstanceOf(XingEntityGeoCode.class);
    XingEntityGeoCode *geoCode = location.geoCode;
    expect(geoCode.accuracy).to.equal(@12);
    expect(geoCode.latitude).to.equal(@53.558572);
    expect(geoCode.longitude).to.equal(@9.927822);

    NSDictionary *companyLinks = @{
    @"xing": @"http://www.xing.com/companies/xingag-qadept.",
    @"thumbnail": @"http://www.xing.com/img/xing/xe/corporate_pages/default_logo_80x25.png",
    @"logo": @"http://www.xing.com/img/xing/xe/corporate_pages/default-logo.png",
    };
    expect(job.company).to.beInstanceOf(XingEntityCompany.class);
    XingEntityCompany *company = job.company;
    expect(company.companyID).to.equal(@"61700_4cae15");
    expect(company.name).to.equal(@"XING AG");
    expect(company.links).to.equal(companyLinks);
}

@end
