//
//  XingUserMappingTest.m
//  XingSDK
//
//  Created by Johannes Plunien on 12/13/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingUserMappingTest.h"

@implementation XingUserMappingTest

- (void)testUserMapping {
    RKEntityMapping *mapping = [self mappingForClass:[XingEntityUser class]];
    NSDictionary *data = [self mappingData:@"user"][@"users"][0];
    RKMappingTest *test = [self testForMapping:mapping withData:data];
    [test performMapping];

    NSArray *premiumServices = @[@"SEARCH", @"PRIVATEMESSAGES"];
    NSArray *badges = @[@"PREMIUM", @"MODERATOR"];
    NSArray *qualifications = @[@"TOEFLS", @"PADI AOWD"];
    NSDictionary *languages = @{
    @"de": @"NATIVE",
    @"en": @"FLUENT",
    @"fr": @"BASIC",
    @"zh": @"BASIC",
    };
    NSDictionary *webProfiles = @{
    @"qype": @[@"http://qype.de/users/foo"],
    @"google_plus": @[@"http://plus.google.com/foo"],
    @"blog": @[@"http://blog.example.org"],
    @"homepage": @[@"http://example.org", @"http://other-example.org"],
    };
    NSDictionary *instantMessagingAccount = @{
    @"skype": @"1122334455",
    @"googletalk": @"max.mustermann",
    };
    XingEntityUser *user = test.destinationObject;

    expect(user.userID).to.equal(@"123456_abcdef");
    expect(user.firstName).to.equal(@"Max");
    expect(user.lastName).to.equal(@"Mustermann");
    expect(user.displayName).to.equal(@"Max Mustermann");
    expect(user.pageName).to.equal(@"Max_Mustermann");
    expect(user.permalink).to.equal(@"https://www.xing.com/profile/Max_Mustermann");
    expect(user.gender).to.equal(@"m");
    expect(user.activeEmail).to.equal(@"max.mustermann@xing.com");
    expect(user.premiumServices).to.equal(premiumServices);
    expect(user.badges).to.equal(badges);
    expect(user.wants).to.equal(@"einen neuen Job");
    expect(user.haves).to.equal(@"viele tolle Skills");
    expect(user.interests).to.equal(@"Flitzebogen schießen and so on");
    expect(user.organisationMember).to.equal(@"ACM, GI");
    expect(user.languages).to.equal(languages);
    expect(user.webProfiles).to.equal(webProfiles);
    expect(user.instantMessagingAccounts).to.equal(instantMessagingAccount);

    expect(user.birthDate).to.beInstanceOf(XingEntityBirthDate.class);
    expect(user.birthDate.day).to.equal(@12);
    expect(user.birthDate.month).to.equal(@8);
    expect(user.birthDate.year).to.equal(@1963);

    expect(user.timeZone).to.beInstanceOf(XingEntityTimeZone.class);
    expect(user.timeZone.name).to.equal(@"Europe/Copenhagen");
    expect(user.timeZone.utcOffset).to.equal(@2.0);

    expect(user.privateAddress).to.beInstanceOf(XingEntityAddressPrivate.class);
    expect(user.privateAddress.city).to.equal(@"Hamburg");
    expect(user.privateAddress.country).to.equal(@"DE");
    expect(user.privateAddress.zipCode).to.equal(@"20357");
    expect(user.privateAddress.street).to.equal(@"Privatstraße 1");
    expect(user.privateAddress.phone).to.equal(@"49|40|1234560");
    expect(user.privateAddress.fax).to.equal(@"||");
    expect(user.privateAddress.province).to.equal(@"Hamburg");
    expect(user.privateAddress.email).to.equal(@"max@mustermann.de");
    expect(user.privateAddress.mobilePhone).to.equal(@"49|0155|1234567");

    expect(user.businessAddress).to.beInstanceOf(XingEntityAddressBusiness.class);
    expect(user.businessAddress.city).to.equal(@"Hamburg");
    expect(user.businessAddress.country).to.equal(@"DE");
    expect(user.businessAddress.zipCode).to.equal(@"20357");
    expect(user.businessAddress.street).to.equal(@"Geschäftsstraße 1a");
    expect(user.businessAddress.phone).to.equal(@"49|40|1234569");
    expect(user.businessAddress.fax).to.equal(@"49|40|1234561");
    expect(user.businessAddress.province).to.equal(@"Hamburg");
    expect(user.businessAddress.email).to.equal(@"max.mustermann@xing.com");
    expect(user.businessAddress.mobilePhone).to.equal(@"49|160|66666661");

    XingEntityAward *award = user.professionalExperience.awards.anyObject;
    expect(user.professionalExperience).to.beInstanceOf(XingEntityProfessionalExperience.class);
    expect(user.professionalExperience.awards.anyObject).to.beInstanceOf(XingEntityAward.class);
    expect(award.dateAwarded).to.equal(@2007);
    expect(award.name).to.equal(@"Awesome Dude Of The Year");
    expect(award.url).to.equal(@"http://www.some-award.com");

    XingEntityUserCompany *company = user.professionalExperience.otherCompanies.anyObject;
    expect(user.professionalExperience.otherCompanies.anyObject).to.beInstanceOf(XingEntityUserCompany.class);
    expect(company.name).to.equal(@"Uni");
    expect(company.title).to.equal(@"Wiss. Mitarbeiter");
    expect(company.companySize).to.equal(@"4000");
    expect(company.tag).to.equal(@"OFFIS");
    expect(company.url).to.equal(@"http://www.uni.de");
    expect(company.careerLevel).to.equal(@"EXPERIENCED");
    expect(company.beginDate).to.equal(@"2007");
    expect(company.companyDescription).to.equal(@"bla");
    expect(company.endDate).to.equal(@"2008");
    expect(company.industry).to.equal(@"APPAREL_AND_FASHION");

    expect(user.professionalExperience.primaryCompany).to.beInstanceOf(XingEntityUserCompany.class);
    expect(user.professionalExperience.primaryCompany.name).to.equal(@"XING AG");
    expect(user.professionalExperience.primaryCompany.title).to.equal(@"Softwareentwickler");
    expect(user.professionalExperience.primaryCompany.companySize).to.equal(@"201-500");
    expect(user.professionalExperience.primaryCompany.tag).to.equal(@"foo-tag");
    expect(user.professionalExperience.primaryCompany.url).to.equal(@"http://www.xing.com");
    expect(user.professionalExperience.primaryCompany.careerLevel).to.equal(@"PROFESSIONAL_EXPERIENCED");
    expect(user.professionalExperience.primaryCompany.beginDate).to.equal(@"2010-01");
    expect(user.professionalExperience.primaryCompany.companyDescription).to.equal(@"foo-description");
    expect(user.professionalExperience.primaryCompany.endDate).to.equal(@"2013-01");
    expect(user.professionalExperience.primaryCompany.industry).to.equal(@"AEROSPACE");

    expect(user.educationalBackground).to.beInstanceOf(XingEntityEducationalBackground.class);
    expect(user.educationalBackground.qualifications).to.equal(qualifications);

    XingEntitySchool *school = user.educationalBackground.schools.anyObject;
    expect(user.educationalBackground.schools.anyObject).to.beInstanceOf(XingEntitySchool.class);
    expect(school.name).to.equal(@"Carl-von-Ossietzky Universtät Schellenburg");
    expect(school.degree).to.equal(@"MSc CE/CS");
    expect(school.notes).to.equal(@"foo notes");
    expect(school.subject).to.equal(@"some subject");
    expect(school.beginDate).to.equal(@"1998-08");
    expect(school.endDate).to.equal(@"2005-02");

    expect(user.photoURL).to.beInstanceOf(XingEntityPhotoURL.class);
    expect(user.photoURL.large).to.equal(@"http://www.xing.com/img/users/e/3/d/f94ef165a.123456,1.140x185.jpg");
    expect(user.photoURL.maxiThumb).to.equal(@"http://www.xing.com/img/users/e/3/d/f94ef165a.123456,1.70x93.jpg");
    expect(user.photoURL.mediumThumb).to.equal(@"http://www.xing.com/img/users/e/3/d/f94ef165a.123456,1.57x75.jpg");
    expect(user.photoURL.miniThumb).to.equal(@"http://www.xing.com/img/users/e/3/d/f94ef165a.123456,1.18x24.jpg");
    expect(user.photoURL.thumb).to.equal(@"http://www.xing.com/img/users/e/3/d/f94ef165a.123456,1.30x40.jpg");
}

@end
