//
//  XingSessionTests.m
//  XingSDK
//
//  Created by Johannes Plunien on 11/18/12.
//  Copyright (c) 2013 Johannes Plunien. All rights reserved.
//

#import "XingSessionTests.h"
#import "XingSession.h"

@interface XingSessionTests ()

@property (nonatomic, strong) XingSession *session;

@end

@implementation XingSessionTests

- (void)setUp {
    [super setUp];
    self.session = [[XingSession alloc] initWithBaseURL:@""];
}

- (void)tearDown {
    self.session = nil;
    [super tearDown];
}

- (void)testIsReady {
    STAssertFalse([self.session isReady], @"Session is not ready without oAuth secrets");
    self.session.consumerSecret = @"consumerSecret";
    STAssertFalse([self.session isReady], @"Session is not ready without oAuth secrets");
    self.session.consumerKey = @"consumerKey";
    STAssertFalse([self.session isReady], @"Session is not ready without oAuth secrets");
    self.session.accessToken = @"accessToken";
    STAssertFalse([self.session isReady], @"Session is not ready without oAuth secrets");
    self.session.accessTokenSecret = @"accessTokenSecret";
    STAssertFalse([self.session isReady], @"Session is not ready without oAuth secrets");
    self.session.xingUserID = @"xingUserID";
    STAssertTrue([self.session isReady], @"Session is ready after setting oAuth secrets");
}

- (void)testNoClientKeyAssertion {
    self.session.consumerKey = nil;
    STAssertThrows([self.session authViewController:^(BOOL authorized, NSError *error) {
    }], @"Throws exception without consumerKey");
}

- (void)testNoClientSecretAssertion {
    self.session.consumerKey = @"consumerKey";
    self.session.consumerSecret = nil;
    STAssertThrows([self.session authViewController:^(BOOL authorized, NSError *error) {
    }], @"Throws exception without consumerSecret");
}

- (void)testAuthViewController {
    self.session.consumerKey = @"consumerKey";
    self.session.consumerSecret = @"consumerSecret";
    STAssertNoThrow([self.session authViewController:^(BOOL authorized, NSError *error) {
    }], @"Throws no exception with consumerKey and consumerSecret");
}

@end
