require 'json'

class XingSDKServer < Sinatra::Base
  self.app_file = __FILE__
  enable :logging

  before do
    content_type 'application/json'
  end

  def send_fixture(*args)
    path = File.join(File.dirname(__FILE__), 'fixtures', *args)
    query = request.env['rack.request.query_string']
    if query.length > 0
      path += '.'
      path += query.gsub(/[^\d\w]+/, '_')
    end
    path += '.json'
    unless File.exists?(path)
      status 404
      return
    end
    send_file path
  end

  delete '/v1/users/123456_abcdef/bookmarks/467437519_1714c5' do
    status 204
  end

  delete '/v1/users/123456_abcdef/conversations/467437519_1714c5/messages/18600191_ea2e1c/read' do
    status 204
  end

  delete '/v1/users/123456_abcdef/conversations/467437519_1714c5' do
    status 204
  end

  delete '/v1/users/123456_abcdef/network/recommendations/user/467437519_1714c5' do
    status 204
  end

  post '/v1/users/123456_abcdef/status_message' do
    status 201
    "Status update has been posted"
  end

  post '/v1/users/123456_abcdef/contact_requests' do
    status 201
    "Contact request was successfully sent"
  end

  post '/v1/users/123456_abcdef/profile_message' do
    status 204
    "Status update has been posted"
  end

  post '/v1/users/18600191_ea2e1c/visits' do
    status 201
    "The profile visit was created successfully."
  end

  post '/v1/activities/8060_317a54/share' do
    status 204
  end

  post '/v1/activities/8060_317a54/comments' do
    status 201
    "The comment was created successfully"
  end

  delete '/v1/activities/8060_317a54' do
    status 204
  end

  delete '/v1/activities/8060_317a54/comments/9050_333a54' do
    status 204
  end

  delete '/v1/activities/8060_317a54/like' do
    status 204
  end

  put '/v1/users/123456_abcdef/bookmarks/467437519_1714c5' do
    status 204
  end

  put '/v1/activities/8060_317a54/like' do
    status 204
  end

  put '/v1/users/123456_abcdef/geo_location' do
    status 204
  end

  put '/v1/users/123456_abcdef/conversations/467437519_1714c5/read' do
    status 204
  end

  put '/v1/users/123456_abcdef/conversations/467437519_1714c5/participants/18600191_ea2e1c' do
    status 204
  end

  put '/v1/users/123456_abcdef/conversations/467437519_1714c5/messages/18600191_ea2e1c/read' do
    status 204
  end

  get '/*' do
    send_fixture('get', params[:splat])
  end

  post '/*' do
    send_fixture('post', params[:splat])
  end
end
